@@ -91,7 +91,7 @@ class RailsEnvironment
     unless plugin.nil?
       plugin.install
     else
-      puts "plugin not found: #{name_uri_or_plugin}"
+      puts "Plugin not found: #{name_uri_or_plugin}"
     end
   end
  
@@ -162,6 +162,10 @@ class Plugin
     @uri =~ /svn(?:\+ssh)?:\/\/*/
   end
   
+  def git_url?
+    @uri =~ /^git:\/\// || @uri =~ /\.git$/
+  end
+  
   def installed?
     File.directory?("#{rails_env.root}/vendor/plugins/#{name}") \
       or rails_env.externals.detect{ |name, repo| self.uri == repo }
@@ -169,7 +173,10 @@ class Plugin
   
   def install(method=nil, options = {})
     method ||= rails_env.best_install_method?
-    method   = :export if method == :http and svn_url?
+    if :http == method
+      method = :export if svn_url?
+      method = :clone  if git_url?
+    end
 
     uninstall if installed? and options[:force]
 
@@ -214,12 +221,12 @@ class Plugin
 
     def run_install_hook
       install_hook_file = "#{rails_env.root}/vendor/plugins/#{name}/install.rb"
-      load install_hook_file if File.exists? install_hook_file
+      load install_hook_file if File.exist? install_hook_file
     end
 
     def run_uninstall_hook
       uninstall_hook_file = "#{rails_env.root}/vendor/plugins/#{name}/uninstall.rb"
-      load uninstall_hook_file if File.exists? uninstall_hook_file
+      load uninstall_hook_file if File.exist? uninstall_hook_file
     end
 
     def install_using_export(options = {})
@@ -239,14 +246,18 @@ class Plugin
 
     def install_using_http(options = {})
       root = rails_env.root
-      mkdir_p "#{root}/vendor/plugins"
-      Dir.chdir "#{root}/vendor/plugins" do
+      mkdir_p "#{root}/vendor/plugins/#{@name}"
+      Dir.chdir "#{root}/vendor/plugins/#{@name}" do
         puts "fetching from '#{uri}'" if $verbose
-        fetcher = RecursiveHTTPFetcher.new(uri)
+        fetcher = RecursiveHTTPFetcher.new(uri, -1)
         fetcher.quiet = true if options[:quiet]
         fetcher.fetch
       end
     end
+    
+    def install_using_clone(options = {})
+      git_command :clone, options
+    end
 
     def svn_command(cmd, options = {})
       root = rails_env.root
@@ -257,12 +268,23 @@ class Plugin
       puts base_cmd if $verbose
       system(base_cmd)
     end
+    
+    def git_command(cmd, options = {})
+      root = rails_env.root
+      mkdir_p "#{root}/vendor/plugins"
+      base_cmd = "git #{cmd} --depth 1 #{uri} \"#{root}/vendor/plugins/#{name}\""
+      puts base_cmd if $verbose
+      puts "removing: #{root}/vendor/plugins/#{name}/.git"
+      system(base_cmd)
+      rm_rf "#{root}/vendor/plugins/#{name}/.git"
+    end
 
     def guess_name(url)
       @name = File.basename(url)
       if @name == 'trunk' || @name.empty?
         @name = File.basename(File.dirname(url))
       end
+      @name.gsub!(/\.git$/, '') if @name =~ /\.git$/
     end
     
     def rails_env
@@ -447,6 +469,8 @@ module Commands
         o.separator "    #{@script_name} install continuous_builder\n"
         o.separator "  Install a plugin from a subversion URL:"
         o.separator "    #{@script_name} install http://dev.rubyonrails.com/svn/rails/plugins/continuous_builder\n"
+        o.separator "  Install a plugin from a git URL:"
+        o.separator "    #{@script_name} install git://github.com/SomeGuy/my_awesome_plugin.git\n"
         o.separator "  Install a plugin and add a svn:externals entry to vendor/plugins"
         o.separator "    #{@script_name} install -x continuous_builder\n"
         o.separator "  List all available plugins:"
@@ -515,7 +539,7 @@ module Commands
         o.on(         "--local", 
                       "List locally installed plugins.") {|@local| @remote = false}
         o.on(         "--remote",
-                      "List remotely availabled plugins. This is the default behavior",
+                      "List remotely available plugins. This is the default behavior",
                       "unless --local is provided.") {|@remote|}
       end
     end
@@ -725,6 +749,9 @@ module Commands
         o.on(         "-o", "--checkout",
                       "Use svn checkout to grab the plugin.",
                       "Enables updating but does not add a svn:externals entry.") { |v| @method = :checkout }
+        o.on(         "-e", "--export",
+                      "Use svn export to grab the plugin.",
+                      "Exports the plugin, allowing you to check it into your local repository. Does not enable updates, or add an svn:externals entry.") { |v| @method = :export }
         o.on(         "-q", "--quiet",
                       "Suppresses the output from installation.",
                       "Ignored if -v is passed (./script/plugin -v install ...)") { |v| @options[:quiet] = true }
@@ -765,8 +792,9 @@ module Commands
       args.each do |name|
         ::Plugin.find(name).install(install_method, @options)
       end
-    rescue
+    rescue StandardError => e
       puts "Plugin not found: #{args.inspect}"
+      puts e.inspect if $verbose
       exit 1
     end
   end
@@ -853,7 +881,8 @@ end
  
 class RecursiveHTTPFetcher
   attr_accessor :quiet
-  def initialize(urls_to_fetch, cwd = ".")
+  def initialize(urls_to_fetch, level = 1, cwd = ".")
+    @level = level
     @cwd = cwd
     @urls_to_fetch = urls_to_fetch.to_a
     @quiet = false
@@ -884,7 +913,8 @@ class RecursiveHTTPFetcher
     links = []
     contents.scan(/href\s*=\s*\"*[^\">]*/i) do |link|
       link = link.sub(/href="/i, "")
-      next if link =~ /^http/i || link =~ /^\./
+      next if link =~ /svnindex.xsl$/
+      next if link =~ /^(\w*:|)\/\// || link =~ /^\./
       links << File.join(base_url, link)
     end
     links
@@ -906,12 +936,14 @@ class RecursiveHTTPFetcher
   end
   
   def fetch_dir(url)
-    push_d(File.basename(url))
+    @level += 1
+    push_d(File.basename(url)) if @level > 0
     open(url) do |stream|
       contents =  stream.read
       fetch(links(url, contents))
     end
-    pop_d
+    pop_d if @level > 0
+    @level -= 1
   end
 end
 
