@@ -1,11 +1,61 @@
 require 'logger'
 require 'set'
-require File.join(File.dirname(__FILE__), 'railties_path')
-require File.join(File.dirname(__FILE__), 'rails/version')
+require 'pathname'
+
+$LOAD_PATH.unshift File.dirname(__FILE__)
+require 'railties_path'
+require 'rails/version'
+require 'rails/plugin/locator'
+require 'rails/plugin/loader'
+require 'rails/gem_dependency'
+
 
 RAILS_ENV = (ENV['RAILS_ENV'] || 'development').dup unless defined?(RAILS_ENV)
 
 module Rails
+  class << self
+    # The Configuration instance used to configure the Rails environment
+    def configuration
+      @@configuration
+    end
+  
+    def configuration=(configuration)
+      @@configuration = configuration
+    end
+  
+    def logger
+      RAILS_DEFAULT_LOGGER
+    end
+  
+    def root
+      if defined?(RAILS_ROOT)
+        RAILS_ROOT
+      else
+        nil
+      end
+    end
+  
+    def env
+      RAILS_ENV
+    end
+  
+    def cache
+      RAILS_CACHE
+    end
+
+    def version
+      VERSION::STRING
+    end
+
+    def public_path
+      @@public_path ||= self.root ? File.join(self.root, "public") : "public"
+    end
+
+    def public_path=(path)
+      @@public_path = path
+    end
+  end
+  
   # The Initializer is responsible for processing the Rails configuration, such
   # as setting the $LOAD_PATH, requiring the right frameworks, initializing
   # logging, and more. It can be run either as a single command that'll just
@@ -17,7 +67,7 @@ module Rails
   # through the block running:
   #
   #   Rails::Initializer.run do |config|
-  #     config.frameworks -= [ :action_web_service ]
+  #     config.frameworks -= [ :action_mailer ]
   #   end
   #
   # This will use the default configuration options from Rails::Configuration,
@@ -28,7 +78,7 @@ module Rails
 
     # The set of loaded plugins.
     attr_reader :loaded_plugins
-
+    
     # Runs the initializer. By default, this will invoke the #process method,
     # which simply executes all of the initialization routines. Alternately,
     # you can specify explicitly which initialization routine you want:
@@ -52,66 +102,60 @@ module Rails
     end
 
     # Sequentially step through all of the available initialization routines,
-    # in order:
-    #
-    # * #set_load_path
-    # * #set_connection_adapters
-    # * #require_frameworks
-    # * #load_environment
-    # * #initialize_database
-    # * #initialize_logger
-    # * #initialize_framework_logging
-    # * #initialize_framework_views
-    # * #initialize_dependency_mechanism
-    # * #initialize_breakpoints
-    # * #initialize_whiny_nils
-    # * #initialize_framework_settings
-    # * #load_environment
-    # * #load_plugins
-    # * #load_observers
-    # * #initialize_routing
-    #
-    # (Note that #load_environment is invoked twice, once at the start and
-    # once at the end, to support the legacy configuration style where the
-    # environment could overwrite the defaults directly, instead of via the
-    # Configuration instance.
+    # in order (view execution order in source).
     def process
-      check_ruby_version
-      set_load_path
-      set_connection_adapters
+      Rails.configuration = configuration
 
+      check_ruby_version
+      install_gem_spec_stubs
+      set_load_path
+      
       require_frameworks
       set_autoload_paths
+      add_gem_load_paths
+      add_plugin_load_paths
       load_environment
 
       initialize_encoding
       initialize_database
+
+      initialize_cache
+      initialize_framework_caches
+
       initialize_logger
       initialize_framework_logging
+
       initialize_framework_views
       initialize_dependency_mechanism
-      initialize_breakpoints
       initialize_whiny_nils
-      initialize_temporary_directories
+      initialize_temporary_session_directory
+      initialize_time_zone
       initialize_framework_settings
 
-      # Support for legacy configuration style where the environment
-      # could overwrite anything set from the defaults/global through
-      # the individual base class configurations.
-      load_environment
-
       add_support_load_paths
 
+      load_gems
       load_plugins
 
-      # Observers are loaded after plugins in case Observers or observed models are modified by plugins.
-      load_observers
+      # pick up any gems that plugins depend on
+      add_gem_load_paths
+      load_gems
+      check_gem_dependencies
+      
+      load_application_initializers
+
+      # the framework is now fully initialized
+      after_initialize
+
+      # Prepare dispatcher callbacks and run 'prepare' callbacks
+      prepare_dispatcher
 
       # Routing must be initialized after plugins to allow the former to extend the routes
       initialize_routing
 
-      # the framework is now fully initialized
-      after_initialize
+      # Observers are loaded after plugins in case Observers or observed models are modified by plugins.
+      
+      load_observers
     end
 
     # Check for valid Ruby version
@@ -121,6 +165,30 @@ module Rails
       require 'ruby_version_check'
     end
 
+    # If Rails is vendored and RubyGems is available, install stub GemSpecs
+    # for Rails, Active Support, Active Record, Action Pack, Action Mailer, and
+    # Active Resource. This allows Gem plugins to depend on Rails even when
+    # the Gem version of Rails shouldn't be loaded.
+    def install_gem_spec_stubs
+      unless Rails.respond_to?(:vendor_rails?)
+        abort %{Your config/boot.rb is outdated: Run "rake rails:update".}
+      end
+
+      if Rails.vendor_rails?
+        begin; require "rubygems"; rescue LoadError; return; end
+
+        stubs = %w(rails activesupport activerecord actionpack actionmailer activeresource)
+        stubs.reject! { |s| Gem.loaded_specs.key?(s) }
+
+        stubs.each do |stub|
+          Gem.loaded_specs[stub] = Gem::Specification.new do |s|
+            s.name = stub
+            s.version = Rails::VERSION::STRING
+          end
+        end
+      end
+    end
+
     # Set the <tt>$LOAD_PATH</tt> based on the value of
     # Configuration#load_paths. Duplicates are removed.
     def set_load_path
@@ -147,61 +215,90 @@ module Rails
       configuration.load_once_paths.freeze
     end
 
-    # Sets the +RAILS_CONNECTION_ADAPTERS+ constant based on the value of
-    # Configuration#connection_adapters. This constant is used to determine
-    # which database adapters should be loaded (by default, all adapters are
-    # loaded).
-    def set_connection_adapters
-      Object.const_set("RAILS_CONNECTION_ADAPTERS", configuration.connection_adapters) if configuration.connection_adapters
-    end
-
     # Requires all frameworks specified by the Configuration#frameworks
-    # list. By default, all frameworks (ActiveRecord, ActiveSupport,
-    # ActionPack, ActionMailer, and ActionWebService) are loaded.
+    # list. By default, all frameworks (Active Record, Active Support,
+    # Action Pack, Action Mailer, and Active Resource) are loaded.
     def require_frameworks
       configuration.frameworks.each { |framework| require(framework.to_s) }
+    rescue LoadError => e
+      # re-raise because Mongrel would swallow it
+      raise e.to_s
     end
 
     # Add the load paths used by support functions such as the info controller
     def add_support_load_paths
     end
 
+    # Adds all load paths from plugins to the global set of load paths, so that
+    # code from plugins can be required (explicitly or automatically via Dependencies).
+    def add_plugin_load_paths
+      plugin_loader.add_plugin_load_paths
+    end
+
+    def add_gem_load_paths
+      unless @configuration.gems.empty?
+        require "rubygems"
+        @configuration.gems.each &:add_load_paths
+      end
+    end
+
+    def load_gems
+      @configuration.gems.each(&:load)
+    end
+
+    def check_gem_dependencies
+      unloaded_gems = @configuration.gems.reject { |g| g.loaded? }
+      if unloaded_gems.size > 0
+        @gems_dependencies_loaded = false
+        # don't print if the gems rake tasks are being run
+        unless $rails_gem_installer
+          puts %{These gems that this application depends on are missing:}
+          unloaded_gems.each do |gem|
+            puts " - #{gem.name}"
+          end
+          puts %{Run "rake gems:install" to install them.}
+        end
+      else
+        @gems_dependencies_loaded = true
+      end
+    end
+
     # Loads all plugins in <tt>config.plugin_paths</tt>.  <tt>plugin_paths</tt>
     # defaults to <tt>vendor/plugins</tt> but may also be set to a list of
     # paths, such as
-    #   config.plugin_paths = ['lib/plugins', 'vendor/plugins']
+    #   config.plugin_paths = ["#{RAILS_ROOT}/lib/plugins", "#{RAILS_ROOT}/vendor/plugins"]
     #
-    # Each plugin discovered in <tt>plugin_paths</tt> is initialized:
-    # * add its +lib+ directory, if present, to the beginning of the load path
-    # * evaluate <tt>init.rb</tt> if present
+    # In the default implementation, as each plugin discovered in <tt>plugin_paths</tt> is initialized:
+    # * its +lib+ directory, if present, is added to the load path (immediately after the applications lib directory)
+    # * <tt>init.rb</tt> is evaluated, if present
     #
     # After all plugins are loaded, duplicates are removed from the load path.
-    # If an array of plugin names is specified in config.plugins, the plugins
-    # will be loaded in that order. Otherwise, plugins are loaded in alphabetical
+    # If an array of plugin names is specified in config.plugins, only those plugins will be loaded
+    # and they plugins will be loaded in that order. Otherwise, plugins are loaded in alphabetical
     # order.
+    #
+    # if config.plugins ends contains :all then the named plugins will be loaded in the given order and all other
+    # plugins will be loaded in alphabetical order
     def load_plugins
-      if configuration.plugins.nil?
-        # a nil value implies we don't care about plugins; load 'em all in a reliable order
-        find_plugins(configuration.plugin_paths).sort.each { |path| load_plugin path }
-      elsif !configuration.plugins.empty?
-        # we've specified a config.plugins array, so respect that order
-        plugin_paths = find_plugins(configuration.plugin_paths)
-        configuration.plugins.each do |name|
-          path = plugin_paths.find { |p| File.basename(p) == name }
-          raise(LoadError, "Cannot find the plugin '#{name}'!") if path.nil?
-          load_plugin path
-        end
-      end 
-      $LOAD_PATH.uniq!
+      plugin_loader.load_plugins
+    end
+
+    def plugin_loader
+      @plugin_loader ||= configuration.plugin_loader.new(self)
     end
 
     # Loads the environment specified by Configuration#environment_path, which
-    # is typically one of development, testing, or production.
+    # is typically one of development, test, or production.
     def load_environment
       silence_warnings do
+        return if @environment_loaded
+        @environment_loaded = true
+        
         config = configuration
         constants = self.class.constants
+        
         eval(IO.read(configuration.environment_path), binding, configuration.environment_path)
+        
         (self.class.constants - constants).each do |const|
           Object.const_set(const, self.class.const_get(const))
         end
@@ -209,14 +306,20 @@ module Rails
     end
 
     def load_observers
-      ActiveRecord::Base.instantiate_observers
+      if @gems_dependencies_loaded && configuration.frameworks.include?(:active_record)
+        ActiveRecord::Base.instantiate_observers
+      end
     end
 
-    # This initialzation sets $KCODE to 'u' to enable the multibyte safe operations.
-    # Plugin authors supporting other encodings should override this behaviour and
-    # set the relevant +default_charset+ on ActionController::Base
+    # For Ruby 1.8, this initialization sets $KCODE to 'u' to enable the
+    # multibyte safe operations. Plugin authors supporting other encodings
+    # should override this behaviour and set the relevant +default_charset+
+    # on ActionController::Base.
+    #
+    # For Ruby 1.9, this does nothing. Specify the default encoding in the Ruby
+    # shebang line if you don't want UTF-8.
     def initialize_encoding
-      $KCODE='u'
+      $KCODE='u' if RUBY_VERSION < '1.9'
     end
 
     # This initialization routine does nothing unless <tt>:active_record</tt>
@@ -224,12 +327,25 @@ module Rails
     # this sets the database configuration from Configuration#database_configuration
     # and then establishes the connection.
     def initialize_database
-      return unless configuration.frameworks.include?(:active_record)
-      ActiveRecord::Base.configurations = configuration.database_configuration
-      ActiveRecord::Base.establish_connection
+      if configuration.frameworks.include?(:active_record)
+        ActiveRecord::Base.configurations = configuration.database_configuration
+        ActiveRecord::Base.establish_connection
+      end
     end
 
-    # If the +RAILS_DEFAULT_LOGGER+ constant is already set, this initialization
+    def initialize_cache
+      unless defined?(RAILS_CACHE)
+        silence_warnings { Object.const_set "RAILS_CACHE", ActiveSupport::Cache.lookup_store(configuration.cache_store) }
+      end
+    end
+
+    def initialize_framework_caches
+      if configuration.frameworks.include?(:action_controller)
+        ActionController::Base.cache_store ||= RAILS_CACHE
+      end
+    end
+
+    # If the RAILS_DEFAULT_LOGGER constant is already set, this initialization
     # routine does nothing. If the constant is not set, and Configuration#logger
     # is not +nil+, this also does nothing. Otherwise, a new logger instance
     # is created at Configuration#log_path, with a default log level of
@@ -243,11 +359,15 @@ module Rails
 
       unless logger = configuration.logger
         begin
-          logger = Logger.new(configuration.log_path)
-          logger.level = Logger.const_get(configuration.log_level.to_s.upcase)
-        rescue StandardError
-          logger = Logger.new(STDERR)
-          logger.level = Logger::WARN
+          logger = ActiveSupport::BufferedLogger.new(configuration.log_path)
+          logger.level = ActiveSupport::BufferedLogger.const_get(configuration.log_level.to_s.upcase)
+          if configuration.environment == "production"
+            logger.auto_flushing = false
+            logger.set_non_blocking_io
+          end
+        rescue StandardError => e
+          logger = ActiveSupport::BufferedLogger.new(STDERR)
+          logger.level = ActiveSupport::BufferedLogger::WARN
           logger.warn(
             "Rails Error: Unable to access log file. Please ensure that #{configuration.log_path} exists and is chmod 0666. " +
             "The log level has been raised to WARN and the output directed to STDERR until the problem is fixed."
@@ -258,32 +378,34 @@ module Rails
       silence_warnings { Object.const_set "RAILS_DEFAULT_LOGGER", logger }
     end
 
-    # Sets the logger for ActiveRecord, ActionController, and ActionMailer
+    # Sets the logger for Active Record, Action Controller, and Action Mailer
     # (but only for those frameworks that are to be loaded). If the framework's
     # logger is already set, it is not changed, otherwise it is set to use
-    # +RAILS_DEFAULT_LOGGER+.
+    # RAILS_DEFAULT_LOGGER.
     def initialize_framework_logging
       for framework in ([ :active_record, :action_controller, :action_mailer ] & configuration.frameworks)
         framework.to_s.camelize.constantize.const_get("Base").logger ||= RAILS_DEFAULT_LOGGER
       end
+      
+      RAILS_CACHE.logger ||= RAILS_DEFAULT_LOGGER
     end
 
-    # Sets the +template_root+ for ActionController::Base and ActionMailer::Base
+    # Sets +ActionController::Base#view_paths+ and +ActionMailer::Base#template_root+
     # (but only for those frameworks that are to be loaded). If the framework's
-    # +template_root+ has already been set, it is not changed, otherwise it is
+    # paths have already been set, it is not changed, otherwise it is
     # set to use Configuration#view_path.
     def initialize_framework_views
-      for framework in ([ :action_controller, :action_mailer ] & configuration.frameworks)
-        framework.to_s.camelize.constantize.const_get("Base").template_root ||= configuration.view_path
-      end
+      ActionMailer::Base.template_root ||= configuration.view_path  if configuration.frameworks.include?(:action_mailer)
+      ActionController::Base.view_paths = [configuration.view_path] if configuration.frameworks.include?(:action_controller) && ActionController::Base.view_paths.empty?
     end
 
-    # If ActionController is not one of the loaded frameworks (Configuration#frameworks)
+    # If Action Controller is not one of the loaded frameworks (Configuration#frameworks)
     # this does nothing. Otherwise, it loads the routing definitions and sets up
     # loading module used to lazily load controllers (Configuration#controller_paths).
     def initialize_routing
       return unless configuration.frameworks.include?(:action_controller)
       ActionController::Routing.controller_paths = configuration.controller_paths
+      ActionController::Routing::Routes.configuration_file = configuration.routes_configuration_file
       ActionController::Routing::Routes.reload
     end
 
@@ -293,26 +415,31 @@ module Rails
       Dependencies.mechanism = configuration.cache_classes ? :require : :load
     end
 
-    # Sets the +BREAKPOINT_SERVER_PORT+ if Configuration#breakpoint_server
-    # is true.
-    def initialize_breakpoints
-      silence_warnings { Object.const_set("BREAKPOINT_SERVER_PORT", 42531) if configuration.breakpoint_server }
-    end
-
     # Loads support for "whiny nil" (noisy warnings when methods are invoked
     # on +nil+ values) if Configuration#whiny_nils is true.
     def initialize_whiny_nils
       require('active_support/whiny_nil') if configuration.whiny_nils
     end
 
-    def initialize_temporary_directories
+    def initialize_temporary_session_directory
       if configuration.frameworks.include?(:action_controller)
-        session_path = "#{RAILS_ROOT}/tmp/sessions/"
+        session_path = "#{configuration.root_path}/tmp/sessions/"
         ActionController::Base.session_options[:tmpdir] = File.exist?(session_path) ? session_path : Dir::tmpdir
+      end
+    end
 
-        cache_path = "#{RAILS_ROOT}/tmp/cache/"
-        if File.exist?(cache_path)
-          ActionController::Base.fragment_cache_store = :file_store, cache_path
+    # Sets the default value for Time.zone, and turns on ActiveRecord::Base#time_zone_aware_attributes.
+    # If assigned value cannot be matched to a TimeZone, an exception will be raised.
+    def initialize_time_zone
+      if configuration.time_zone
+        zone_default = Time.send!(:get_zone, configuration.time_zone)
+        unless zone_default
+          raise %{Value assigned to config.time_zone not recognized. Run "rake -D time" for a list of tasks for finding appropriate time zone names.}
+        end
+        Time.zone_default = zone_default
+        if configuration.frameworks.include?(:active_record)
+          ActiveRecord::Base.time_zone_aware_attributes = true
+          ActiveRecord::Base.default_timezone = :utc
         end
       end
     end
@@ -328,80 +455,34 @@ module Rails
           base_class.send("#{setting}=", value)
         end
       end
+      configuration.active_support.each do |setting, value|
+        ActiveSupport.send("#{setting}=", value)
+      end
     end
 
     # Fires the user-supplied after_initialize block (Configuration#after_initialize)
     def after_initialize
-      configuration.after_initialize_block.call if configuration.after_initialize_block
+      if @gems_dependencies_loaded
+        configuration.after_initialize_blocks.each do |block|
+          block.call
+        end
+      end
     end
 
-    protected
-      # Return a list of plugin paths within base_path.  A plugin path is
-      # a directory that contains either a lib directory or an init.rb file.
-      # This recurses into directories which are not plugin paths, so you
-      # may organize your plugins within the plugin path.
-      def find_plugins(*base_paths)
-        base_paths.flatten.inject([]) do |plugins, base_path|
-          Dir.glob(File.join(base_path, '*')).each do |path|
-            if plugin_path?(path)
-              plugins << path if plugin_enabled?(path)
-            elsif File.directory?(path)
-              plugins += find_plugins(path)
-            end
-          end
-          plugins
+    def load_application_initializers
+      if @gems_dependencies_loaded
+        Dir["#{configuration.root_path}/config/initializers/**/*.rb"].sort.each do |initializer|
+          load(initializer)
         end
       end
+    end
 
-      def plugin_path?(path)
-        File.directory?(path) and (File.directory?(File.join(path, 'lib')) or File.file?(File.join(path, 'init.rb')))
-      end
+    def prepare_dispatcher
+      require 'dispatcher' unless defined?(::Dispatcher)
+      Dispatcher.define_dispatcher_callbacks(configuration.cache_classes)
+      Dispatcher.new(RAILS_DEFAULT_LOGGER).send :run_callbacks, :prepare_dispatch
+    end
 
-      def plugin_enabled?(path)
-        configuration.plugins.nil? || configuration.plugins.include?(File.basename(path))
-      end
-
-      # Load the plugin at <tt>path</tt> unless already loaded.
-      #
-      # Each plugin is initialized:
-      # * add its +lib+ directory, if present, to the beginning of the load path
-      # * evaluate <tt>init.rb</tt> if present
-      #
-      # Returns <tt>true</tt> if the plugin is successfully loaded or
-      # <tt>false</tt> if it is already loaded (similar to Kernel#require).
-      # Raises <tt>LoadError</tt> if the plugin is not found.
-      def load_plugin(directory)
-        name = File.basename(directory)
-        return false if loaded_plugins.include?(name)
-
-        # Catch nonexistent and empty plugins.
-        raise LoadError, "No such plugin: #{directory}" unless plugin_path?(directory)
-
-        lib_path  = File.join(directory, 'lib')
-        init_path = File.join(directory, 'init.rb')
-        has_lib   = File.directory?(lib_path)
-        has_init  = File.file?(init_path)
-
-        # Add lib to load path *after* the application lib, to allow
-        # application libraries to override plugin libraries.
-        if has_lib
-          application_lib_index = $LOAD_PATH.index(File.join(RAILS_ROOT, "lib")) || 0
-          $LOAD_PATH.insert(application_lib_index + 1, lib_path)
-          Dependencies.load_paths << lib_path
-          Dependencies.load_once_paths << lib_path
-        end
-
-        # Allow plugins to reference the current configuration object
-        config = configuration
-	
-        # Add to set of loaded plugins before 'name' collapsed in eval.
-        loaded_plugins << name
-
-        # Evaluate init.rb.
-        silence_warnings { eval(IO.read(init_path), binding, init_path) } if has_init
-
-        true
-      end
   end
 
   # The Configuration class holds all the parameters for the Initializer and
@@ -414,33 +495,31 @@ module Rails
   #   config = Rails::Configuration.new
   #   Rails::Initializer.run(:process, config)
   class Configuration
-    # A stub for setting options on ActionController::Base
+    # The application's base directory.
+    attr_reader :root_path
+
+    # A stub for setting options on ActionController::Base.
     attr_accessor :action_controller
 
-    # A stub for setting options on ActionMailer::Base
+    # A stub for setting options on ActionMailer::Base.
     attr_accessor :action_mailer
 
-    # A stub for setting options on ActionView::Base
+    # A stub for setting options on ActionView::Base.
     attr_accessor :action_view
 
-    # A stub for setting options on ActionWebService::Base
-    attr_accessor :action_web_service
-
-    # A stub for setting options on ActiveRecord::Base
+    # A stub for setting options on ActiveRecord::Base.
     attr_accessor :active_record
 
-    # Whether or not to use the breakpoint server (boolean)
-    attr_accessor :breakpoint_server
+    # A stub for setting options on ActiveRecord::Base.
+    attr_accessor :active_resource
+
+    # A stub for setting options on ActiveSupport.
+    attr_accessor :active_support
 
     # Whether or not classes should be cached (set to false if you want
     # application classes to be reloaded on each request)
     attr_accessor :cache_classes
 
-    # The list of connection adapters to load. (By default, all connection
-    # adapters are loaded. You can set this to be just the adapter(s) you
-    # will use to reduce your application's load time.)
-    attr_accessor :connection_adapters
-
     # The list of paths that should be searched for controllers. (Defaults
     # to <tt>app/controllers</tt> and <tt>components</tt>.)
     attr_accessor :controller_paths
@@ -448,11 +527,15 @@ module Rails
     # The path to the database configuration file to use. (Defaults to
     # <tt>config/database.yml</tt>.)
     attr_accessor :database_configuration_file
+    
+    # The path to the routes configuration file to use. (Defaults to
+    # <tt>config/routes.rb</tt>.)
+    attr_accessor :routes_configuration_file
 
     # The list of rails framework components that should be loaded. (Defaults
     # to <tt>:active_record</tt>, <tt>:action_controller</tt>,
     # <tt>:action_view</tt>, <tt>:action_mailer</tt>, and
-    # <tt>:action_web_service</tt>).
+    # <tt>:active_resource</tt>).
     attr_accessor :frameworks
 
     # An array of additional paths to prepend to the load path. By default,
@@ -478,6 +561,9 @@ module Rails
     # used directly.
     attr_accessor :logger
 
+    # The specific cache store to use. By default, the ActiveSupport::Cache::Store will be used.
+    attr_accessor :cache_store
+
     # The root of the application's views. (Defaults to <tt>app/views</tt>.)
     attr_accessor :view_path
 
@@ -488,15 +574,83 @@ module Rails
     # The list of plugins to load. If this is set to <tt>nil</tt>, all plugins will
     # be loaded. If this is set to <tt>[]</tt>, no plugins will be loaded. Otherwise,
     # plugins will be loaded in the order specified.
-    attr_accessor :plugins
+    attr_reader :plugins
+    def plugins=(plugins)
+      @plugins = plugins.nil? ? nil : plugins.map { |p| p.to_sym }
+    end
 
     # The path to the root of the plugins directory. By default, it is in
     # <tt>vendor/plugins</tt>.
     attr_accessor :plugin_paths
 
+    # The classes that handle finding the desired plugins that you'd like to load for
+    # your application. By default it is the Rails::Plugin::FileSystemLocator which finds
+    # plugins to load in <tt>vendor/plugins</tt>. You can hook into gem location by subclassing
+    # Rails::Plugin::Locator and adding it onto the list of <tt>plugin_locators</tt>.
+    attr_accessor :plugin_locators
+
+    # The class that handles loading each plugin. Defaults to Rails::Plugin::Loader, but
+    # a sub class would have access to fine grained modification of the loading behavior. See
+    # the implementation of Rails::Plugin::Loader for more details.
+    attr_accessor :plugin_loader
+    
+    # Enables or disables plugin reloading.  You can get around this setting per plugin.
+    # If <tt>reload_plugins?</tt> is false, add this to your plugin's <tt>init.rb</tt>
+    # to make it reloadable:
+    #
+    #   Dependencies.load_once_paths.delete lib_path
+    #
+    # If <tt>reload_plugins?</tt> is true, add this to your plugin's <tt>init.rb</tt>
+    # to only load it once:
+    #
+    #   Dependencies.load_once_paths << lib_path
+    #
+    attr_accessor :reload_plugins
+
+    # Returns true if plugin reloading is enabled.
+    def reload_plugins?
+      !!@reload_plugins
+    end
+
+    # An array of gems that this rails application depends on.  Rails will automatically load
+    # these gems during installation, and allow you to install any missing gems with:
+    #
+    #   rake gems:install
+    #
+    # You can add gems with the #gem method.
+    attr_accessor :gems
+
+    # Adds a single Gem dependency to the rails application.
+    #
+    #   # gem 'aws-s3', '>= 0.4.0'
+    #   # require 'aws/s3'
+    #   config.gem 'aws-s3', :lib => 'aws/s3', :version => '>= 0.4.0', \
+    #     :source => "http://code.whytheluckystiff.net"
+    #
+    def gem(name, options = {})
+      @gems << Rails::GemDependency.new(name, options)
+    end
+    
+    # Deprecated options:
+    def breakpoint_server(_ = nil)
+      $stderr.puts %(
+      *******************************************************************
+      * config.breakpoint_server has been deprecated and has no effect. *
+      *******************************************************************
+      )
+    end
+    alias_method :breakpoint_server=, :breakpoint_server
+
+    # Sets the default +time_zone+.  Setting this will enable +time_zone+
+    # awareness for Active Record models and set the Active Record default
+    # timezone to <tt>:utc</tt>.
+    attr_accessor :time_zone
+
     # Create a new Configuration instance, initialized with the default
     # values.
     def initialize
+      set_root_path!
+
       self.frameworks                   = default_frameworks
       self.load_paths                   = default_load_paths
       self.load_once_paths              = default_load_once_paths
@@ -505,15 +659,39 @@ module Rails
       self.view_path                    = default_view_path
       self.controller_paths             = default_controller_paths
       self.cache_classes                = default_cache_classes
-      self.breakpoint_server            = default_breakpoint_server
       self.whiny_nils                   = default_whiny_nils
       self.plugins                      = default_plugins
       self.plugin_paths                 = default_plugin_paths
+      self.plugin_locators              = default_plugin_locators
+      self.plugin_loader                = default_plugin_loader
       self.database_configuration_file  = default_database_configuration_file
+      self.routes_configuration_file    = default_routes_configuration_file
+      self.gems                         = default_gems
 
       for framework in default_frameworks
         self.send("#{framework}=", Rails::OrderedOptions.new)
       end
+      self.active_support = Rails::OrderedOptions.new
+    end
+
+    # Set the root_path to RAILS_ROOT and canonicalize it.
+    def set_root_path!
+      raise 'RAILS_ROOT is not set' unless defined?(::RAILS_ROOT)
+      raise 'RAILS_ROOT is not a directory' unless File.directory?(::RAILS_ROOT)
+
+      @root_path =
+        # Pathname is incompatible with Windows, but Windows doesn't have
+        # real symlinks so File.expand_path is safe.
+        if RUBY_PLATFORM =~ /(:?mswin|mingw)/
+          File.expand_path(::RAILS_ROOT)
+
+        # Otherwise use Pathname#realpath which respects symlinks.
+        else
+          Pathname.new(::RAILS_ROOT).realpath.to_s
+        end
+      
+      Object.const_set(:RELATIVE_RAILS_ROOT, ::RAILS_ROOT.dup) unless defined?(::RELATIVE_RAILS_ROOT)
+      ::RAILS_ROOT.replace @root_path
     end
 
     # Loads and returns the contents of the #database_configuration_file. The
@@ -523,28 +701,28 @@ module Rails
       YAML::load(ERB.new(IO.read(database_configuration_file)).result)
     end
 
-    # The path to the current environment's file (development.rb, etc.). By
+    # The path to the current environment's file (<tt>development.rb</tt>, etc.). By
     # default the file is at <tt>config/environments/#{environment}.rb</tt>.
     def environment_path
       "#{root_path}/config/environments/#{environment}.rb"
     end
 
     # Return the currently selected environment. By default, it returns the
-    # value of the +RAILS_ENV+ constant.
+    # value of the RAILS_ENV constant.
     def environment
       ::RAILS_ENV
     end
 
-    # Sets a block which will be executed after rails has been fully initialized.
+    # Adds a block which will be executed after rails has been fully initialized.
     # Useful for per-environment configuration which depends on the framework being
     # fully initialized.
     def after_initialize(&after_initialize_block)
-      @after_initialize_block = after_initialize_block
+      after_initialize_blocks << after_initialize_block if after_initialize_block
     end
 
-    # Returns the block set in Configuration#after_initialize
-    def after_initialize_block
-      @after_initialize_block
+    # Returns the blocks added with Configuration#after_initialize
+    def after_initialize_blocks
+      @after_initialize_blocks ||= []
     end
 
     # Add a preparation callback that will run before every request in development
@@ -552,8 +730,10 @@ module Rails
     #
     # See Dispatcher#to_prepare.
     def to_prepare(&callback)
-      require 'dispatcher' unless defined?(::Dispatcher)
-      Dispatcher.to_prepare(&callback)
+      after_initialize do 
+        require 'dispatcher' unless defined?(::Dispatcher)
+        Dispatcher.to_prepare(&callback)
+      end
     end
 
     def builtin_directories
@@ -562,33 +742,30 @@ module Rails
     end
 
     def framework_paths
-      # TODO: Don't include dirs for frameworks that are not used
-      %w(
-        railties
-        railties/lib
-        actionpack/lib
-        activesupport/lib
-        activerecord/lib
-        actionmailer/lib
-        actionwebservice/lib
-      ).map { |dir| "#{framework_root_path}/#{dir}" }.select { |dir| File.directory?(dir) }
+      paths = %w(railties railties/lib activesupport/lib)
+      paths << 'actionpack/lib' if frameworks.include? :action_controller or frameworks.include? :action_view
+      
+      [:active_record, :action_mailer, :active_resource, :action_web_service].each do |framework|
+        paths << "#{framework.to_s.gsub('_', '')}/lib" if frameworks.include? framework
+      end
+      
+      paths.map { |dir| "#{framework_root_path}/#{dir}" }.select { |dir| File.directory?(dir) }
     end
 
     private
-      def root_path
-        ::RAILS_ROOT
-      end
-
       def framework_root_path
         defined?(::RAILS_FRAMEWORK_ROOT) ? ::RAILS_FRAMEWORK_ROOT : "#{root_path}/vendor/rails"
       end
 
       def default_frameworks
-        [ :active_record, :action_controller, :action_view, :action_mailer, :action_web_service ]
+        [ :active_record, :action_controller, :action_view, :action_mailer, :active_resource ]
       end
 
       def default_load_paths
-        paths = ["#{root_path}/test/mocks/#{environment}"]
+        paths = []
+        
+        # Add the old mock paths only if the directories exists
+        paths.concat(Dir["#{root_path}/test/mocks/#{environment}"]) if File.exists?("#{root_path}/test/mocks/#{environment}")
 
         # Add the app's controller directory
         paths.concat(Dir["#{root_path}/app/controllers/"])
@@ -603,7 +780,6 @@ module Rails
           app/controllers
           app/helpers
           app/services
-          app/apis
           components
           config
           lib
@@ -630,12 +806,16 @@ module Rails
         File.join(root_path, 'config', 'database.yml')
       end
 
+      def default_routes_configuration_file
+        File.join(root_path, 'config', 'routes.rb')
+      end
+
       def default_view_path
         File.join(root_path, 'app', 'views')
       end
 
       def default_controller_paths
-        paths = [ File.join(root_path, 'app', 'controllers'), File.join(root_path, 'components') ]
+        paths = [File.join(root_path, 'app', 'controllers')]
         paths.concat builtin_directories
         paths
       end
@@ -648,10 +828,6 @@ module Rails
         false
       end
 
-      def default_breakpoint_server
-        false
-      end
-
       def default_whiny_nils
         false
       end
@@ -663,12 +839,34 @@ module Rails
       def default_plugin_paths
         ["#{root_path}/vendor/plugins"]
       end
+
+      def default_plugin_locators
+        locators = []
+        locators << Plugin::GemLocator if defined? Gem
+        locators << Plugin::FileSystemLocator
+      end
+
+      def default_plugin_loader
+        Plugin::Loader
+      end
+      
+      def default_cache_store
+        if File.exist?("#{root_path}/tmp/cache/")
+          [ :file_store, "#{root_path}/tmp/cache/" ]
+        else
+          :memory_store
+        end
+      end
+      
+      def default_gems
+        []
+      end
   end
 end
 
 # Needs to be duplicated from Active Support since its needed before Active
 # Support is available. Here both Options and Hash are namespaced to prevent
-# conflicts with other implementations AND with the classes residing in ActiveSupport.
+# conflicts with other implementations AND with the classes residing in Active Support.
 class Rails::OrderedOptions < Array #:nodoc:
   def []=(key, value)
     key = key.to_sym
