@@ -1,4 +1,4 @@
-require File.dirname(__FILE__) + '<%= '/..' * class_nesting_depth %>/../test_helper'
+require 'test_helper'
 
 class <%= controller_class_name %>ControllerTest < ActionController::TestCase
   def test_should_get_index
